const {app, session, protocol, shell, BrowserWindow, globalShortcut, Menu} = require('electron');
const path = require('path');

const appVersion = "1.4.1"; // VERSION_UPDATE Always change this Version Number
const requestPromise = require('minimal-request-promise');
const os = require('os');

let mainWindow = null;
let loadingWindow = null;
let serverProcess = null;
let updateWindow = null;
let errorWindow = null;

const headers = {
        'license':'0abdea078047b12df42e7750ccba34d69b-CommunityEdition'
};
const options = {
    extraHeaders:"license:"+headers.license
};
const versionUrl = 'https://licenses.xeotek.io/v1/version';
const appUrl = 'http://localhost:8133';
let uID;
const hasJavaHome = !!process.env.JAVA_HOME;
const openLoadingWindow = function(){
    loadingWindow = new BrowserWindow({
        title: 'KaDeck',
        width: 400,
        height: 270,
        frame: false,
        resizable: false,
        backgroundColor: "#009ae1",
        show:false
    });
    loadingWindow.loadFile("index.html");

    loadingWindow.on("closed", () => {
        loadingWindow = null;
    });

    loadingWindow.on("ready-to-show", ()=>{
        loadingWindow.show();
    });
};

const openErrorDialog = function(cause){
    errorWindow = new BrowserWindow({
        title: 'An error occurred!',
        width: 400,
        height: 170,
        frame: true,
        resizable: false,
        backgroundColor: "#f3f3f3",
        alwaysOnTop:true
    });
    errorWindow.loadFile("error.html");
    errorWindow.on("closed", () => {
        errorWindow = null;
    });
};

const verifyJava = function(){
    console.log("JAVA_HOME "+(hasJavaHome?"found.":"not found."));
    let java;

    if (hasJavaHome){
        console.log("Trying to start Java using JAVA_HOME.");
        try{
            let path = "";
            if (process.env.JAVA_HOME.toString().endsWith("bin/")){
                path = process.env.JAVA_HOME + 'java';
            }else if (process.env.JAVA_HOME.toString().endsWith("bin")){
                path = process.env.JAVA_HOME + '/java';
            }else{
                path = process.env.JAVA_HOME.toString().endsWith("/")?process.env.JAVA_HOME + 'bin/java':process.env.JAVA_HOME + '/bin/java';
            }
            java = require('child_process')
                .spawnSync(path, ["-version"]);
        }catch(e){}
        if (java && !java.error){
            return true;
        }else{
            console.log("Could not start "+path+".");
        }
    }
    console.log("Trying using 'java' if set.");
    try {
        java = require('child_process')
            .spawnSync('java', ["-version"]);
    }catch(e){}

    return !!(java && !java.error);
};

function compareVersion(v1, v2) {
    if (typeof v1 !== 'string') return false;
    if (typeof v2 !== 'string') return false;
    v1 = v1.split('.');
    v2 = v2.split('.');
    const k = Math.min(v1.length, v2.length);
    for (let i = 0; i < k; ++ i) {
        v1[i] = parseInt(v1[i], 10);
        v2[i] = parseInt(v2[i], 10);
        if (v1[i] > v2[i]) return 1;
        if (v1[i] < v2[i]) return -1;
    }
    return v1.length === v2.length ? 0: (v1.length < v2.length ? -1 : 1);
}

function checkForUpdates(){
    const req = {
        headers:{
            'platform':os.platform(),
            'variant':'community-edition',
            'version':appVersion,
            'ci':uID
        }
    };
    requestPromise.get(versionUrl,req).then(function (response) {

        let latest_version = JSON.parse(response.body).latest_version;

        if (compareVersion(appVersion,latest_version) < 0){

            updateWindow = new BrowserWindow({
              title: 'Newer Version!',
              width: 400,
              height: 170,
              frame: true,
              resizable: false,
              backgroundColor: "#f3f3f3",
                alwaysOnTop:true
          });
          updateWindow.loadFile("version.html");
          updateWindow.on("closed", () => {
              updateWindow = null;
          });
        }
    }, function (err) {
        return;
    });
}

// Provide API for web application
global.callElectronUiApi = function (args) {
    console.log('Electron called from web app with args "' + args + '"');

    if (args) {
        if (args[0] === 'exit') {
            exit();
        }

        if (args[0] === 'minimize') {
            mainWindow.minimize();
        }

        if (args[0] === 'maximize') {
            if (!mainWindow.isMaximized()) {
                mainWindow.maximize();
            } else {
                mainWindow.unmaximize();
            }
        }
    }
};

function exit() {
    console.log('Stopping KaDeck...');

    const kill = require('tree-kill');
    kill(serverProcess.pid, 'SIGTERM', function (err) {
        console.log('KaDeck stopped.');

        serverProcess = null;
        mainWindow.close();
        if (loadingWindow!=null) {
            loadingWindow.close();
        }
    });
}

app.on('window-all-closed', function () {
    app.quit();
});

app.on('ready', function () {
    console.log("###### ###### ###### GREETINGS FROM XEOTEK ###### ###### ######");
    console.log("Dear User,");
    console.log("What is going on here? Something is wrong?");
    console.log("If you are seeing this because you have encountered a problem while using KaDeck, please feel free to contact us: support@kadeck.com.");
    console.log("Greetings from Xeotek Team.");
    console.log("###### ###### ###### X E O T E K ###### ###### ######");

    const storage = require('electron-localstorage');
    storage.setStoragePath(app.getPath('userData')+"/sets.bt");
    if (! storage.getItem("$a")){
        storage.setItem('$a', uuidv4());
    }
    uID = storage.getItem("$a");
    checkForUpdates();

    if (!verifyJava()){
        console.log("Dear User, it seems your Java configuration is \"corrupt\". We could not start java. Try java -version to verify Java is installed correctly. We use JAVA_HOME. Contact us for help (support@kadeck.com).");
        openErrorDialog("java");
        return;
    }

    openLoadingWindow();

    if(os.platform() === "darwin"){
        var template = [{
            label: "Application",
            submenu: [
                { label: "KaDeck Community Edition "+appVersion },
                { type: "separator" },
                { label: "Quit", accelerator: "Command+Q", click: function() { app.quit(); }}
            ]}, {
            label: "Edit",
            submenu: [
                { label: "Undo", accelerator: "CmdOrCtrl+Z", selector: "undo:" },
                { label: "Redo", accelerator: "Shift+CmdOrCtrl+Z", selector: "redo:" },
                { type: "separator" },
                { label: "Cut", accelerator: "CmdOrCtrl+X", selector: "cut:" },
                { label: "Copy", accelerator: "CmdOrCtrl+C", selector: "copy:" },
                { label: "Paste", accelerator: "CmdOrCtrl+V", selector: "paste:" },
                { label: "Select All", accelerator: "CmdOrCtrl+A", selector: "selectAll:" }
            ]}
        ];
        
        Menu.setApplicationMenu(Menu.buildFromTemplate(template));
    }
    platform = process.platform;

    if (platform === 'win32') {
        serverProcess = require('child_process')
            .spawn(app.getAppPath() + '/electron-kadeck/bin/KaDeck.bat');
    } else {
        serverProcess = require('child_process')
            .spawn(app.getAppPath() + '/electron-kadeck/bin/KaDeck');
    }

    if (!serverProcess) {
        console.error('Unable to start KaDeck from ' + app.getAppPath());
        app.quit();
        return;
    }

    serverProcess.stdout.on('data', function (data) {
        console.log('KaDeck: ' + data);
    });

    console.log("KaDeck PID: " + serverProcess.pid);




    function setupVaadinFilesService() {
        protocol.registerFileProtocol('vaadin', (request, callback) => {
            console.log(`Vaadin Request URL: ${request.url}`);

            let urlPath = request.url.substr('vaadin://'.length);
            if (urlPath.indexOf('?') >= 0) {
                urlPath = urlPath.substr(0, urlPath.indexOf('?'));
            }
            if (urlPath.indexOf('#') >= 0) {
                urlPath = urlPath.substr(0, urlPath.indexOf('#'));
            }
            console.log(`Vaadin Request Path: ${urlPath}`);

            const fsPath = path.normalize(`${__dirname}/electron-kadeck/VAADIN/${urlPath}`);

            console.log(`Vaadin Request File: ${fsPath}`);

            callback({path: fsPath});
        }, (error) => {
            if (error) console.error('Failed to register protocol');
        });

        const filter = {
            urls: ['http://localhost:8133/VAADIN/*']
        };
        session.defaultSession.webRequest.onBeforeRequest(filter, (details, callback) => {
            let vaadinFile = details.url.replace('http://localhost:8133/VAADIN/', '');

            console.log(`Vaadin URL: ${vaadinFile}`);

            callback({cancel: false, redirectURL: 'vaadin://' + vaadinFile})
        });
    }



    const openWindow = function () {
        // setupVaadinFilesService();

        const selectionMenu = Menu.buildFromTemplate([
            {role: 'copy'},
            {type: 'separator'},
            {role: 'selectall'},
        ]);

        const inputMenu = Menu.buildFromTemplate([
            {role: 'cut', accelerator: "CmdOrCtrl+X"},
            {role: 'copy',  accelerator: "CmdOrCtrl+C"},
            {role: 'paste', accelerator: "CmdOrCtrl+V"},
            {type: 'separator'},
            {role: 'selectall', accelerator: "CmdOrCtrl+A"},
        ]);

        mainWindow = new BrowserWindow({
            title: 'KaDeck',
            width: 1280,
            height: 768,
            frame: true,
            backgroundColor: "#fbfbfb",
            icon: path.join(__dirname, 'assets/icons/icon'),
            show:false
        });



        mainWindow.loadURL(appUrl,options);
        mainWindow.webContents.on('context-menu', (e, props) => {
            const { selectionText, isEditable } = props;
            if (isEditable) {
                inputMenu.popup(mainWindow);
            } else if (selectionText && selectionText.trim() !== '') {
                selectionMenu.popup(mainWindow);
            }
        });

        mainWindow.webContents.session.clearCache(()=>{});
        // mainWindow.loadFile('index.html');
        // uncomment to show debug tools
        //  mainWindow.webContents.openDevTools();

        mainWindow.on('closed', function () {
            mainWindow = null;
        });

        mainWindow.on('show', ()=>{
        });


        mainWindow.webContents.on('new-window', function(event, url){
            event.preventDefault();
            shell.openExternal(url+"?ci="+uID);
        });

        mainWindow.on('close', function (e) {
            if (serverProcess) {
                e.preventDefault();
                exit();
            }
        });

        mainWindow.on("ready-to-show", ()=>{
            mainWindow.loadURL(appUrl,options);
            mainWindow.show();
            if (loadingWindow!=null) {
                loadingWindow.close();
            }
        });


    };
    let count = 0;
    const maxCount = 30;
    const startUp = function () {
        const req = {
            headers:{
                'license':'0abdea078047b12df42e7750ccba34d69b-CommunityEdition'
            }
        };
        requestPromise.get(appUrl,req).then(function (response) {
            console.log('Server started!');

            setTimeout(function(){
                openWindow();
            }, 300);
        }, function (response) {
            console.log('Waiting for the server start...');

            setTimeout(function () {
                count++;
                // if (count > maxCount) {
                     //TODO implement error handling
                //     return;
                // }
                startUp();
            }, 200);
        });
    };

    startUp();

    // Register a shortcut listener.
    const ret = globalShortcut.register('CommandOrControl+Shift+`', () => {
        console.log('Bring to front shortcut triggered');
        if (mainWindow) {
            mainWindow.focus();
        }
    })
});

app.on('will-quit', () => {
    // Unregister all shortcuts.
    globalShortcut.unregisterAll();
});


function uuidv4() {
    return 'xxxxxyyyxxxyy'.replace(/[xy]/g, function(c) {
        var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
        return v.toString(20)+v.toString(10);
    });
}
